% slvlprms 08-Jul-2018 

% retrieve raw observed sea level data and atmospheric pressure data
%   apply inverse barometer correction and low pass filter
% save to mat file

% basic initial setup
wipe;
modprelims;

% retrieve inputs
ingrps = { 'in'; }; inputsdiary;

% initialize gap-free timestamps 
hrss.dnest = datenum(min(yr),1,1):(1/24):datenum(max(yr),12,31,23,30,0);
lp12.dnest = datenum(min(yr),1,1):0.5:datenum(max(yr),12,31,13,0,0);

% loop over stations
for istn = 1:length(stn.name)
    fprintf('working on station: %s\n',stn.name{istn});
    %% first, do water level
    fprintf('   working on water level %s\n',stn.name{istn});
    filpthnam_allyr = sprintf('%s%sSeaLevel%dto%d.mat',wldir,...
        stn.name{istn},min(yr),max(yr));
    if exist(filpthnam_allyr,'file') ~=2 | forceretrieve == 1
        wl = [];
        dnest = [];
        % loop over years
        for iyr = 1:length(yr)
            fprintf('      working on year %d\n',yr(iyr));
            filpthnam = sprintf('%s%s%s%s',wldir,stn.name{istn},...
                string(yr(iyr)),'v2.txt');
            % if it's not yet on disk, go retrieve it
            if exist(filpthnam,'file') ~= 2 | forceretrieve == 1
                tmp = urlread(sprintf('%s%s%s%d%s%d%s',wlurl.p1,stn.id{istn},...
                    wlurl.p2,yr(iyr),wlurl.p3,yr(iyr),wlurl.p4)); 
                fp = fopen(filpthnam,'w');
                fprintf(fp,tmp);
                fclose(fp);
                clear fp tmp;
            end
            % read in this year's data and append to global vector
            fp = fopen(filpthnam);
            dat = textscan(fp,'%s %s %f %f %s %f %s %f %f %s','HeaderLines',2);
            wl = [wl; dat{6}];
            for i = 1:length(dat{5})
                dnest = [dnest; ...
                    datenum(dat{5}{i},'yyyy-mm-ddTHH:MM:SSZ')-(5/24)];
            end
            fclose(fp);
            clear fp dat i filpthnam;
        end
        eval(sprintf('save %s wl dnest;',filpthnam_allyr));
        clear wl dnest;
    end
    load(filpthnam_allyr);
    clear filpthnam_allyr;
    % (wl and dnest are full now)
    %% map to pre-initialized gap-free time grid
    wl = interp1(dnest,wl,hrss.dnest);
    for i = 1:length(wl)
        if isequal(mod(i,1000),0)
            fprintf('      replacing nans: %d of %d\n',i,length(wl));
        end
        if min(abs(hrss.dnest(i)-dnest))>1.5/24
            wl(i) = nan;
        end
    end
    clear dnest;
    %% now do pressure
    fprintf('   working on atm pressure %s\n',stn.name{istn});
    filpthnam_allyr = sprintf('%s%sAtmPres%dto%d.mat',pdir,...
        stn.name{istn},min(yr),max(yr));
    if exist(filpthnam_allyr,'file') ~=2  | forceretrieve == 1
        pr = [];
        dnest_pr = [];
        % loop over years
        for iyr = 1:length(yr)
            fprintf('      working on year %d\n',yr(iyr));
            filpthnam = sprintf('%s%s%s%s',pdir,stn.name{istn},...
                string(yr(iyr)),'v2.txt');
            % if it's not yet on disk, go retrieve it
            if exist(filpthnam,'file') ~= 2 | forceretrieve == 1
                %% create temporary monthly files
                fillst = {};
                % jan - nov
                for i = 1:11
                    tmp = urlread(sprintf('%s%s%s%d-%02d%s%d-%02d%s',purl.p1,...
                        stn.id{istn},purl.p2,yr(iyr),i,purl.p3,yr(iyr),i+1,...
                        purl.p4));
                    if length(tmp) > 1000 % skip months that had no data
                        filpthnam_t = sprintf('%s-%02d.txt',...
                            filpthnam(1:end-4),i);
                        fp = fopen(filpthnam_t,'w');
                        fprintf(fp,tmp);
                        fclose(fp);
                        fillst{length(fillst)+1} = filpthnam_t;
                        clear fp filpthnam_t;
                    end
                    clear tmp;
                end
                clear i;
                % dec
                tmp = urlread(sprintf('%s%s%s%d-%02d%s%d-%02d%s',purl.p1,...
                    stn.id{istn},purl.p2,yr(iyr),12,purl.p3,yr(iyr)+1,1,...
                    purl.p4)); 
                if length(tmp) > 1000
                    filpthnam_t = sprintf('%s-12.txt',filpthnam(1:end-4));
                    fp = fopen(filpthnam_t,'w');
                    fprintf(fp,tmp);
                    fclose(fp);
                    fillst{length(fillst)+1} = filpthnam_t;
                    clear fp filpthnam_t;
                end
                clear tmp filpthnam_t;
                % merge monthly files in to an annual file
                fp = fopen(filpthnam,'w');
                for i = 1:length(fillst)
                    fpmon = fopen(fillst{i},'r');
                    if i ~= 1
                        fgetl(fpmon); fgetl(fpmon);
                    end
                    while ~feof(fpmon)
                        str = fgetl(fpmon);
                        % keep all but final line (1st hour of following year)
                        if isempty(strfind(str,sprintf('%d',yr(iyr)+1)))
                            % only keep top of the hour
                            if strfind(str,':00:00Z')
                                fprintf(fp,[str '\n']);
                            end
                        end
                    end
                    fclose(fpmon);
                    clear fpmon;
                    eval(sprintf('delete %s',fillst{i})); 
                end
                fclose(fp);
                clear fp fillst i;
            end
            % read in this year's data and append to global vector
            fp = fopen(filpthnam);
            dat = textscan(fp,'%s %s %f %f %s %f %s','HeaderLines',2);
            pr = [pr; dat{6}];
            for i = 1:length(dat{5})
                dnest_pr = [dnest_pr; ...
                    datenum(dat{5}{i},'yyyy-mm-ddTHH:MM:SSZ')-(5/24)];
            end
            fclose(fp);
            clear fp dat i filpthnam;
        end
        eval(sprintf('save %s pr dnest_pr;',filpthnam_allyr));
        clear pr dnest_pr;
    end
    load(filpthnam_allyr);
    clear filpthnam_allyr;
    % (now pr and dnest_pr are full)
    %% map to pre-initialized gap-free time grid
    pr = interp1(dnest_pr,pr,hrss.dnest);
    for i = 1:length(pr)
        if isequal(mod(i,1000),0)
            fprintf('      replacing nans: %d of %d\n',i,length(pr));
        end
        if min(abs(hrss.dnest(i)-dnest_pr))>1.5/24
            pr(i) = nan;
        end
    end
    clear dnest_pr;
    %% inverse barometer adjustment
    pr = pr - nanmean(pr);
    awl = wl +sw_dpth(0.01*pr,41);
    clear pr;
    %% lowpass and subsample
    fprintf('   lowpass waterlevel\n');
    wl_lp = tri_var_ss(wl,hrss.dnest,hlfwid,minpct,gaplen,lp12.dnest);
    fprintf('   lowpass atm press\n');
    awl_lp = tri_var_ss(awl,hrss.dnest,hlfwid,minpct,gaplen,lp12.dnest);
    %% rename
    eval(sprintf('hrss.wl_%s = wl;',stn.name{istn}));
    eval(sprintf('hrss.awl_%s = awl;',stn.name{istn}));
    eval(sprintf('lp12.wl_%s = wl_lp;',stn.name{istn}));
    eval(sprintf('lp12.awl_%s = awl_lp;',stn.name{istn}));
    clear wl awl wl_lp awl_lp;
end
%% "prlnp" = providence less newport; "wld" water level difference
% "awld" = adjusted water level difference
hrss.wld_prlnp = hrss.wl_Prov - hrss.wl_Newp;
hrss.awld_prlnp = hrss.awl_Prov - hrss.awl_Newp;
lp12.wld_prlnp = lp12.wl_Prov - lp12.wl_Newp;
lp12.awld_prlnp = lp12.awl_Prov - lp12.awl_Newp;

descr = strvcat(...
    'Sea level parameters; D.Codiga 7/8/2018                        ',...
    'Based on observed sea level and atmos pressure from Providence ',...
    'and Newport stations.                                          ',...
    '   hrss = hourly resolution                                    ',...
    '   lp12 = 25-hr halfwidth triangle-wt lowpass, 12-hr subsampled',...
    '   dnest = datenum eastern standard time                       ',...
    '   wl = water level; awl = inverse-barom. adjusted wat. level  ',...
    '   wld & awld = differences; prlnp = providence less newport   ');
   
save dat/SeaLevels descr hrss lp12;

% end of module
diary off;