% tidprms 08-Jul-2018 

% retrieves tidal predictions
% computes and saves tidal parameters using them
%   including "dymn12" which is daily mean values (of tidal range etc)
%       interpolated to a 12-hour grid

% basic initial setup
wipe;
modprelims;

% retrieve inputs
ingrps = { 'in'; }; inputsdiary;

% initialize gap-free timestamps 
hrss.dnest = datenum(min(yr),1,1):(1/24):datenum(max(yr),12,31,23,30,0);

filpthnam_allyr =sprintf('%sNewpPredHght%dto%d.mat',indir,min(yr),max(yr));
if exist(filpthnam_allyr,'file') ~=2 | forceretrieve == 1
    pred = [];
    dnest = [];
    % loop over years
    for iyr = 1:length(yr)
        fprintf('      working on year %d\n',yr(iyr));
        filpthnam = sprintf('%sNewp%s%s',indir,string(yr(iyr)),'.txt');
        % if it's not yet on disk, go retrieve it
        if exist(filpthnam,'file') ~= 2 | forceretrieve == 1
            tmp = urlread(sprintf('%s%d%s%d%s',url.p1,yr(iyr),url.p2,...
                yr(iyr),url.p3));
            fp = fopen(filpthnam,'w');
            fprintf(fp,tmp);
            fclose(fp);
            clear fp tmp;
        end
        % read in this year's data and append to global vector
        fp = fopen(filpthnam);
        dat = textscan(fp,'%f %f/%f/%f %f:%f %f','HeaderLines',37);
        pred = [pred; dat{7}];
        for i = 1:length(dat{5})
            dnest = [dnest; ...
                datenum(dat{4}(i),dat{2}(i),dat{3}(i),dat{5}(i),...
                    dat{6}(i),0)-(5/24)];
        end
        fclose(fp);
        clear fp dat i filpthnam;
    end
    eval(sprintf('save %s pred dnest;',filpthnam_allyr));
    clear pred dnest;
end
load(filpthnam_allyr);
clear filpthnam_allyr;

% map to pre-initialized full hourly vector of times
pred = interp1(dnest,pred,hrss.dnest);
for i = 1:length(pred)
    if isequal(mod(i,1000),0)
        fprintf('      replacing nans: %d of %d\n',i,length(pred));
    end
    if min(abs(hrss.dnest(i)-dnest))>1.5/24
        pred(i) = nan;
    end
end
clear dnest;

%rename
eta = pred;
clear pred;
dnest = hrss.dnest;
clear hrss;

raw.dnest = dnest;
raw.hght = eta;

% compute ~twice daily values of range
tst = sign(diff(eta));
tst(tst==0) = 1;
tst = diff(tst);
whrmin = find(tst==2)+1;
whrmax = find(tst==-2)+1;
len = min([length(whrmin) length(whrmax)]);
range = eta(whrmax(1:len))-eta(whrmin(1:len));
dnest_range = dnest(whrmax(1:len))+0.5*mode(diff(dnest));
clear tst whrmin whrmax len;

% average those values to get daily range values
dymn.dnest = (floor(min(dnest_range)):floor(max(dnest_range)))+0.5;
[yr1,mn1,dy1] = datevec(dymn.dnest);
[yr2,mn2,dy2] = datevec(dnest_range);
for idy = 1:length(yr1)
    whr = find(yr2==yr1(idy) & mn2==mn1(idy) & dy2==dy1(idy));
    dymn.rng(idy) = mean(range(whr));
    dymn.rngcbd(idy) = mean(range(whr).^3);
end
clear idy whr yr1 yr2 mn1 mn2 dy1 dy2;

% create "dymn12" by interpolating (note the triangle lp is NOT applied)
dymn12.dnest = ceil(min(dymn.dnest)):0.5:floor(max(dymn.dnest));
dymn12.rng = interp1(dymn.dnest,dymn.rng,dymn12.dnest);
dymn12.rngcbd = interp1(dymn.dnest,dymn.rngcbd,dymn12.dnest);

save dat/tidprms raw dymn12;

% end of module
diary off;