% nc2wndprms 09-Oct-2018 

% retrieve NARR winds from the NOAA server using opendap
% compute wind parameters from them
% save them to a mat file

% basic initial setup
wipe;
modprelims;

% retrieve inputs
ingrps = { 'params'; }; inputsdiary;

%% get lat and lon
% get all lat/lons
url = [indir varname{1} monolevel_file{1} ...
            '.' num2str(yrs(1)) '.nc'];
ncid = netcdf.open([url '?lat']);
latall = double(netcdf.getVar(ncid,0));
ncid = netcdf.open([url '?lon']);
lonall = double(netcdf.getVar(ncid,0));

% determine indices of selected lat-lon subregion
ind = 0;
xnum = [];
ynum = [];
for ix = 1:size(latall,2)
    for iy = 1:size(latall,1)
        if latall(iy,ix)<latmax & latall(iy,ix)>latmin ...
                & lonall(iy,ix)<lonmax & lonall(iy,ix)>lonmin
            ind = ind+1;
            xnum = [xnum ix];
            ynum = [ynum iy];
        end
    end
end
xind = min(xnum):max(xnum);
yind = min(ynum):max(ynum);
clear ind ix iy xnum ynum;

% keep only lat-lon values from subregion
lat = latall(yind,xind);
lon = lonall(yind,xind);
clear latall lonall;

%% this loop retrieves uwnd, vwnd, dnest
for ivar = 1:length(varname)
    fprintf('working on variable %s (%d of %d)\n',...
        varname{ivar},ivar,length(varname));

    for iyr = 1:length(yrs)
        fprintf(' working on year %d of %d\n',iyr,length(yrs));

        url = [indir varname{ivar} monolevel_file{ivar} ...
            '.' num2str(yrs(iyr)) '.nc'];

        % get times
        ncid = netcdf.open([url '?time']);
        dnest1 = double(netcdf.getVar(ncid,0));
        dnest1 = datenum(1800,1,1,0,0,0)+ dnest1/24 - (5/24);
        dnest1 = dnest1';

        % get varb values from subregion only
        ncid = netcdf.open([url '?' varname{ivar} ...
                    sprintf('[0:%d][%d:%d][%d:%d]',...
            length(dnest1)-1,min(yind),max(yind),min(xind),max(xind)) ]);
        eval(sprintf('result1 = double(netcdf.getVar(ncid,0));'));
        clear ncid url; 
        
        % 2. shiftdim to match previous (gingko) runs 
        result1 = shiftdim(result1,2);

        % append to multi-year variables
        if iyr>1
            dnest = [dnest dnest1];
            result(size(result,1)+1:size(result,1)+size(result1,1),:,:) ...
                = result1;
        else
            dnest = dnest1;
            result = result1;
        end
        clear result1 dnest1;
    end
    % interpolate to 0/3/6/9... 3hrly grid in dnest
    [tyr,tmon,tday,thr,tmin,tsec] = datevec(dnest(1));
    tgrdmin = datenum(tyr,tmon,tday,thr-mod(thr,3)+3,0,0);
    [tyr,tmon,tday,thr,tmin,tsec] = datevec(dnest(end));
    tgrdmax = datenum(tyr,tmon,tday,thr-mod(thr,3),0,0);
    tgrd = tgrdmin:(3/24):tgrdmax;
    result = interp1(dnest,result,tgrd);
    dnest = tgrd;
    clear tgrd tgrdmin tgrdmax tyr tmon tday thr tmin tsec;
    eval(sprintf('%s = result;',varname{ivar}));
    clear result filnam;
end
clear ivar iyr;

% use those results to compute and save various wind parameters
% save the following:
%   uwnd, vwnd, wndspd, wnddir, wndspdcbd
% for each of:
%   "raw": 3-hr (raw NARR) 
%   "dymn": daily means
%   "lp12": lowpass, 12-hr decimated

% wind speeds are in m/s (at 10 m elevation)
% the winddir values are ccw from east, in radians
% uwnd is eastward, vwnd is northward

% raw
raw.dnest = dnest;
raw.uwnd = uwnd;
raw.vwnd = vwnd;
raw.wndspd = sqrt(uwnd.*uwnd + vwnd.*vwnd);
raw.wnddir = atan2(vwnd,uwnd);
raw.spdcbd = (raw.wndspd).^3;
raw.toN  = uwnd*cos(  90*def.rpd)+vwnd*sin(  90*def.rpd);
raw.toNNE = uwnd*cos(  67.5*def.rpd)+vwnd*sin(  67.5*def.rpd);
raw.toNE = uwnd*cos(  45*def.rpd)+vwnd*sin(  45*def.rpd);
raw.toENE = uwnd*cos(  22.5*def.rpd)+vwnd*sin(  22.5*def.rpd);
raw.toE  = uwnd*cos(   0*def.rpd)+vwnd*sin(   0*def.rpd);
raw.toESE = uwnd*cos(  -22.5*def.rpd)+vwnd*sin(  -22.5*def.rpd);
raw.toSE = uwnd*cos( -45*def.rpd)+vwnd*sin( -45*def.rpd);
raw.toSSE = uwnd*cos(  -67.5*def.rpd)+vwnd*sin(  -67.5*def.rpd);
raw.toS  = uwnd*cos( -90*def.rpd)+vwnd*sin( -90*def.rpd);
raw.toSSW  = uwnd*cos( -112.5*def.rpd)+vwnd*sin( -112.5*def.rpd);
raw.toSW = uwnd*cos(-135*def.rpd)+vwnd*sin(-135*def.rpd);
raw.toWSW = uwnd*cos(-157.5*def.rpd)+vwnd*sin(-157.5*def.rpd);
raw.toW  = uwnd*cos(-180*def.rpd)+vwnd*sin(-180*def.rpd);
raw.toWNW  = uwnd*cos(157.5*def.rpd)+vwnd*sin(157.5*def.rpd);
raw.toNW = uwnd*cos( 135*def.rpd)+vwnd*sin( 135*def.rpd);
raw.toNNW  = uwnd*cos(112.5*def.rpd)+vwnd*sin(112.5*def.rpd);

% dymn
[yr,mn,dy] = datevec(dnest);
stind = min(find(dy==1)); % omits all dates prior to the first of a month
clear yr mn dy;
ind = 0;
for i = stind:8:length(dnest)-7
    ind = ind+1;
    [yr,mn,dy] = datevec(dnest(i));
    dymn.dnest(ind) = datenum(yr,mn,dy)+0.5; % noon
    clear yr mn dy;
    dymn.uwnd(ind) = nanmean(raw.uwnd(i:i+7));
    dymn.vwnd(ind) = nanmean(raw.vwnd(i:i+7));
    dymn.wndspd(ind) = nanmean(raw.wndspd(i:i+7));
    dymn.wnddir(ind) = atan2(dymn.vwnd(ind),dymn.uwnd(ind));
    dymn.spdcbd(ind) = nanmean(raw.spdcbd(i:i+7));
    dymn.toN  = nanmean(raw.toN(i:i+7));
    dymn.toNNE = nanmean(raw.toNNE(i:i+7));
    dymn.toNE = nanmean(raw.toNE(i:i+7));
    dymn.toENE = nanmean(raw.toENE(i:i+7));
    dymn.toE  = nanmean(raw.toE(i:i+7));
    dymn.toESE = nanmean(raw.toESE(i:i+7));
    dymn.toSE = nanmean(raw.toSE(i:i+7));
    dymn.toSSE = nanmean(raw.toSSE(i:i+7));
    dymn.toS  = nanmean(raw.toS(i:i+7));
    dymn.toSSW = nanmean(raw.toSSW(i:i+7))
    dymn.toSW = nanmean(raw.toSW(i:i+7));
    dymn.toWSW = nanmean(raw.toWSW(i:i+7))
    dymn.toW  = nanmean(raw.toW(i:i+7));
    dymn.toWNW = nanmean(raw.toWNW(i:i+7));
    dymn.toNW = nanmean(raw.toNW(i:i+7));
    dymn.toNNW = nanmean(raw.toNNW(i:i+7));
end
clear ind i;

% lp12
lp12.dnest = min(dymn.dnest)+1:0.5:max(dymn.dnest)-1;
lp12.uwnd = tri_var_ss(raw.uwnd,raw.dnest,hlfwid,minpct,gaplen,lp12.dnest);
lp12.vwnd = tri_var_ss(raw.vwnd,raw.dnest,hlfwid,minpct,gaplen,lp12.dnest);
lp12.wndspd = tri_var_ss(raw.wndspd,raw.dnest,hlfwid,minpct,gaplen,...
    lp12.dnest);
lp12.wnddir = atan2(lp12.vwnd,lp12.uwnd);
lp12.spdcbd = tri_var_ss(raw.spdcbd,raw.dnest,hlfwid,minpct,gaplen,...
    lp12.dnest);
lp12.toN = tri_var_ss(raw.toN,raw.dnest,hlfwid,minpct,gaplen,lp12.dnest);
lp12.toNNE = tri_var_ss(raw.toNNE,raw.dnest,hlfwid,minpct,gaplen,lp12.dnest);
lp12.toNE = tri_var_ss(raw.toNE,raw.dnest,hlfwid,minpct,gaplen,lp12.dnest);
lp12.toENE = tri_var_ss(raw.toENE,raw.dnest,hlfwid,minpct,gaplen,lp12.dnest);
lp12.toE = tri_var_ss(raw.toE,raw.dnest,hlfwid,minpct,gaplen,lp12.dnest);
lp12.toESE = tri_var_ss(raw.toESE,raw.dnest,hlfwid,minpct,gaplen,lp12.dnest);
lp12.toSE = tri_var_ss(raw.toSE,raw.dnest,hlfwid,minpct,gaplen,lp12.dnest);
lp12.toSSE = tri_var_ss(raw.toSSE,raw.dnest,hlfwid,minpct,gaplen,lp12.dnest);
lp12.toS = tri_var_ss(raw.toS,raw.dnest,hlfwid,minpct,gaplen,lp12.dnest);
lp12.toSSW = tri_var_ss(raw.toSSW,raw.dnest,hlfwid,minpct,gaplen,lp12.dnest);
lp12.toSW = tri_var_ss(raw.toSW,raw.dnest,hlfwid,minpct,gaplen,lp12.dnest);
lp12.toWSW = tri_var_ss(raw.toWSW,raw.dnest,hlfwid,minpct,gaplen,lp12.dnest);
lp12.toW = tri_var_ss(raw.toW,raw.dnest,hlfwid,minpct,gaplen,lp12.dnest);
lp12.toWNW = tri_var_ss(raw.toWNW,raw.dnest,hlfwid,minpct,gaplen,lp12.dnest);
lp12.toNW = tri_var_ss(raw.toNW,raw.dnest,hlfwid,minpct,gaplen,lp12.dnest);
lp12.toNNW = tri_var_ss(raw.toNNW,raw.dnest,hlfwid,minpct,gaplen,lp12.dnest);

descr = ['wind speed (uwnd eastward, vwnd northward) in m/s; '...
    'wind direction in radians ccw of east'];

eval(sprintf('save dat/%s descr raw dymn lp12 lat lon;',outfilnam));

% end of module
diary off;